"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");
var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");
_Object$defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _reduce = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/reduce"));
var _keys = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/keys"));
var _startsWith = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/starts-with"));
var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));
var _assign = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/assign"));
var _jsxRuntime = require("@emotion/react/jsx-runtime");
var _react = _interopRequireDefault(require("react"));
var _Radio = require("./Radio.context");
var _Checkbox = require("../Checkbox/Checkbox.context");
var _classnames = _interopRequireDefault(require("classnames"));
var _Radio2 = _interopRequireDefault(require("./Radio"));
function getDataOrAriaProps(props) {
  var _context;
  // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
  return (0, _reduce.default)(_context = (0, _keys.default)(props)).call(_context, (prev, key) => {
    if (((0, _startsWith.default)(key).call(key, 'data-') || (0, _startsWith.default)(key).call(key, 'aria-') || key === 'role') && !(0, _startsWith.default)(key).call(key, 'data-__')) {
      // eslint-disable-next-line no-param-reassign
      prev[key] = props[key];
    }
    return prev;
  }, {});
}
/**
 * Same as React.useState but `setState` accept `ignoreDestroy` param to not to setState after destroyed.
 * We do not make this auto is to avoid real memory leak.
 * Developer should confirm it's safe to ignore themselves.
 */
function useSafeState(defaultValue) {
  const destroyRef = _react.default.useRef(false);
  const [value, setValue] = _react.default.useState(defaultValue);
  _react.default.useEffect(() => {
    destroyRef.current = false;
    return () => {
      destroyRef.current = true;
    };
  }, []);
  function safeSetState(updater, ignoreDestroy) {
    if (ignoreDestroy && destroyRef.current) {
      return;
    }
    setValue(updater);
  }
  return [value, safeSetState];
}
const SizeContext = /*#__PURE__*/_react.default.createContext(undefined);
/**
 * Similar to `useState` but will use props value if provided.
 * Note that internal use rc-util `useState` hook.
 */
function useMergedState(defaultStateValue, option) {
  const {
    defaultValue,
    value,
    onChange,
    postState
  } = option || {};
  const [innerValue, setInnerValue] = useSafeState(() => {
    if (value !== undefined) {
      return value;
    }
    if (defaultValue !== undefined) {
      return typeof defaultValue === 'function' ? defaultValue() : defaultValue;
    }
    return typeof defaultStateValue === 'function' ? defaultStateValue() : defaultStateValue;
  });
  let mergedValue = value !== undefined ? value : innerValue;
  if (postState) {
    mergedValue = postState(mergedValue);
  }
  // setState
  const onChangeRef = _react.default.useRef(onChange);
  onChangeRef.current = onChange;
  const triggerChange = _react.default.useCallback((newValue, ignoreDestroy) => {
    setInnerValue(newValue, ignoreDestroy);
    if (mergedValue !== newValue && onChangeRef.current) {
      onChangeRef.current(newValue, mergedValue);
    }
  }, [mergedValue, onChangeRef]);
  // Effect of reset value to `undefined`
  const prevValueRef = _react.default.useRef(value);
  _react.default.useEffect(() => {
    // console.log(value);
    if (value !== undefined && value !== prevValueRef.current) {
      setInnerValue(value);
    }
    prevValueRef.current = value;
  }, [value]);
  return [mergedValue, triggerChange];
}
const RadioGroup = /*#__PURE__*/_react.default.forwardRef((props, ref) => {
  const {
    getPrefixCls,
    direction
  } = _react.default.useContext(_Checkbox.ConfigContext);
  const size = _react.default.useContext(SizeContext);
  const [value, setValue] = useMergedState(props.defaultValue, {
    value: props.value
  });
  const onRadioChange = ev => {
    const lastValue = value;
    const val = ev.target.value;
    if (!('value' in props)) {
      setValue(val);
    }
    const {
      onChange
    } = props;
    if (onChange && val !== lastValue) {
      onChange(ev);
    }
  };
  const renderGroup = () => {
    const {
      prefixCls: customizePrefixCls,
      className = '',
      options,
      disabled,
      children,
      size: customizeSize,
      style,
      id,
      onMouseEnter,
      onMouseLeave
    } = props;
    const prefixCls = getPrefixCls('radio', customizePrefixCls);
    const groupPrefixCls = `${prefixCls}-group`;
    let childrenToRender = children;
    // 如果存在 options, 优先使用
    if (options && options.length > 0) {
      childrenToRender = (0, _map.default)(options).call(options, option => {
        if (typeof option === 'string' || typeof option === 'number') {
          // string 타입 유도
          return (0, _jsxRuntime.jsx)(_Radio2.default, (0, _assign.default)({
            prefixCls: prefixCls,
            disabled: disabled,
            value: option,
            checked: value === option
          }, {
            children: option
          }), option.toString());
        }
        // { label: string value: string } 으로 변형
        return (0, _jsxRuntime.jsx)(_Radio2.default
        // eslint-disable-next-line @typescript-eslint/restrict-template-expressions
        , (0, _assign.default)({
          prefixCls: prefixCls,
          disabled: option.disabled || disabled,
          value: option.value,
          checked: value === option.value,
          style: option.style
        }, {
          children: option.label
        }), `radio-group-value-options-${option.value}`);
      });
    }
    const mergedSize = customizeSize || size;
    const classString = (0, _classnames.default)(groupPrefixCls, `${groupPrefixCls}`, {
      // eslint-disable-next-line @typescript-eslint/restrict-template-expressions
      [`${groupPrefixCls}-${mergedSize}`]: mergedSize,
      [`${groupPrefixCls}-rtl`]: direction === 'rtl'
    }, className);
    return (0, _jsxRuntime.jsx)("div", (0, _assign.default)({}, getDataOrAriaProps(props), {
      className: classString,
      style: style,
      onMouseEnter: onMouseEnter,
      onMouseLeave: onMouseLeave,
      id: id,
      ref: ref
    }, {
      children: childrenToRender
    }));
  };
  return (0, _jsxRuntime.jsx)(_Radio.RadioGroupContextProvider, (0, _assign.default)({
    value: {
      onChange: onRadioChange,
      value,
      disabled: props.disabled,
      name: props.name
    }
  }, {
    children: renderGroup()
  }));
});
var _default = exports.default = /*#__PURE__*/_react.default.memo(RadioGroup);