import _includesInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/includes";
import _Object$assign from "@babel/runtime-corejs3/core-js-stable/object/assign";
import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import { useEffect, useRef, useState } from 'react';
import SelectPanel from './selectPanel';
import { DropdownHeader } from './header';
import { useMultiSelect } from './MultiSelect.hooks';
import * as Styled from './MultiSelect.styled';
import { KEY } from './MultiSelect.constants';
import { LoadingIcon, ArrowIcon, CrossIcon } from './MultiSelect.icons';
import { useDidUpdateEffect, useKey } from '../@hooks';
const MultiSelectSelect = () => {
  const {
    t,
    onMenuToggle,
    ArrowRenderer,
    shouldToggleOnHover,
    isLoading,
    disabled,
    onChange,
    labelledBy,
    value,
    isOpen,
    defaultIsOpen,
    ClearSelectedIcon,
    closeOnChangedValue,
    direction = 'bottom',
    isError
  } = useMultiSelect();
  useEffect(() => {
    if (closeOnChangedValue) setExpanded(false);
  }, [value]);
  const [isInternalExpand, setIsInternalExpand] = useState(true);
  const [expanded, setExpanded] = useState(defaultIsOpen);
  const [hasFocus, setHasFocus] = useState(false);
  const FinalArrow = ArrowRenderer || ArrowIcon;
  const wrapper = useRef();
  useDidUpdateEffect(() => {
    onMenuToggle && onMenuToggle(expanded);
  }, [expanded]);
  useEffect(() => {
    if (defaultIsOpen === undefined && typeof isOpen === 'boolean') {
      setIsInternalExpand(false);
      setExpanded(isOpen);
    }
  }, [isOpen]);
  const handleKeyDown = e => {
    var _context, _context2;
    var _a;
    // allows space and enter when focused on input/button
    if (_includesInstanceProperty(_context = ['text', 'button']).call(_context, e.target.type) && _includesInstanceProperty(_context2 = [KEY.SPACE, KEY.ENTER]).call(_context2, e.code)) {
      return;
    }
    if (isInternalExpand) {
      if (e.code === KEY.ESCAPE) {
        setExpanded(false);
        (_a = wrapper === null || wrapper === void 0 ? void 0 : wrapper.current) === null || _a === void 0 ? void 0 : _a.focus();
      } else {
        setExpanded(true);
      }
    }
    e.preventDefault();
  };
  useKey([KEY.ENTER, KEY.ARROW_DOWN, KEY.SPACE, KEY.ESCAPE], handleKeyDown, {
    target: wrapper
  });
  const handleHover = isExpanded => {
    isInternalExpand && shouldToggleOnHover && setExpanded(isExpanded);
  };
  const handleFocus = () => !hasFocus && setHasFocus(true);
  const handleBlur = e => {
    if (!e.currentTarget.contains(e.relatedTarget) && isInternalExpand) {
      setHasFocus(false);
      setExpanded(false);
    }
  };
  const handleMouseEnter = () => handleHover(true);
  const handleMouseLeave = () => handleHover(false);
  const toggleExpanded = () => {
    isInternalExpand && setExpanded(isLoading || disabled ? false : !expanded);
  };
  const handleClearSelected = e => {
    e.stopPropagation();
    onChange([]);
    isInternalExpand && setExpanded(false);
  };
  return _jsxs(Styled.DSSelectWrapper, _Object$assign({
    tabIndex: 0,
    "aria-labelledby": labelledBy,
    "aria-expanded": expanded,
    "aria-readonly": true,
    "aria-disabled": disabled,
    ref: wrapper,
    "$error": isError,
    onFocus: handleFocus,
    onBlur: handleBlur,
    onMouseEnter: handleMouseEnter,
    onMouseLeave: handleMouseLeave
  }, {
    children: [_jsxs(Styled.SelectHeadArea, _Object$assign({
      onClick: toggleExpanded
    }, {
      children: [_jsx(DropdownHeader, {}), isLoading && _jsx(LoadingIcon, {}), value.length > 0 && ClearSelectedIcon !== null && _jsx("button", _Object$assign({
        type: "button",
        onClick: handleClearSelected,
        disabled: disabled,
        "aria-label": t('clearSelected')
      }, {
        children: ClearSelectedIcon || _jsx(CrossIcon, {})
      })), _jsx(Styled.SelectArrowWrapper, _Object$assign({
        "$isOpen": expanded
      }, {
        children: _jsx(FinalArrow, {})
      }))]
    })), expanded && _jsx(Styled.SelectContent, _Object$assign({
      "$direction": direction
    }, {
      children: _jsx(Styled.PanelContent, {
        children: _jsx(SelectPanel, {})
      })
    }))]
  }));
};
export default MultiSelectSelect;