import _Promise from "@babel/runtime-corejs3/core-js-stable/promise";
import _mapInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/map";
import _filterInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/filter";
import _includesInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/includes";
import _everyInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/every";
import _findIndexInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/find-index";
import _someInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/some";
import _Object$assign from "@babel/runtime-corejs3/core-js-stable/object/assign";
var __awaiter = this && this.__awaiter || function (thisArg, _arguments, P, generator) {
  function adopt(value) {
    return value instanceof P ? value : new P(function (resolve) {
      resolve(value);
    });
  }
  return new (P || (P = _Promise))(function (resolve, reject) {
    function fulfilled(value) {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    }
    function rejected(value) {
      try {
        step(generator["throw"](value));
      } catch (e) {
        reject(e);
      }
    }
    function step(result) {
      result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected);
    }
    step((generator = generator.apply(thisArg, _arguments || [])).next());
  });
};
import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/**
 * This component represents the entire panel which gets dropped down when the
 * user selects the component.  It encapsulates the search filter, the
 * Select-all item, and the list of options.
 */
import { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import * as Styled from '../MultiSelect.styled';
import { useMultiSelect } from '../MultiSelect.hooks';
import { KEY } from '../MultiSelect.constants';
import { simpleFilterOptions } from '../MultiSelect.utils';
import { CrossIcon } from '../MultiSelect.icons';
import SelectItem from '../selectItem';
import SelectList from '../selectList';
import { useKey } from '../../@hooks';
import { debounce } from '../../@utils';
var FocusType;
(function (FocusType) {
  FocusType[FocusType["SEARCH"] = 0] = "SEARCH";
  FocusType[FocusType["NONE"] = -1] = "NONE";
})(FocusType || (FocusType = {}));
const SelectPanel = () => {
  const {
    t,
    onChange,
    options,
    setOptions,
    value,
    filterOptions: customFilterOptions,
    ItemRenderer,
    disabled,
    disableSearch,
    hasSelectAll,
    ClearIcon,
    debounceDuration,
    isCreatable,
    onCreateOption,
    isCheckboxShowing,
    closeOnChangedValue
  } = useMultiSelect();
  const listRef = useRef();
  const searchInputRef = useRef();
  const [searchText, setSearchText] = useState('');
  const [filteredOptions, setFilteredOptions] = useState(options);
  const [searchTextForFilter, setSearchTextForFilter] = useState('');
  const [focusIndex, setFocusIndex] = useState(0);
  const debouncedSearch = useCallback(debounce(query => setSearchTextForFilter(query), debounceDuration), []);
  const skipIndex = useMemo(() => {
    let start = 0;
    if (!disableSearch) start += 1; // if search is enabled then +1 to skipIndex
    if (hasSelectAll) start += 1; // if select-all is enabled then +1 to skipIndex
    return start;
  }, [disableSearch, hasSelectAll]);
  const selectAllOption = {
    label: searchText ? t('selectAllFiltered') : t('selectAll'),
    value: ''
  };
  const selectAllValues = checked => {
    var _context;
    const filteredValues = _mapInstanceProperty(_context = _filterInstanceProperty(filteredOptions).call(filteredOptions, o => !o.disabled)).call(_context, o => o.value);
    if (checked) {
      var _context2;
      const selectedValues = _mapInstanceProperty(value).call(value, o => o.value);
      const finalSelectedValues = [...selectedValues, ...filteredValues];
      return _filterInstanceProperty(_context2 = customFilterOptions ? filteredOptions : options).call(_context2, o => _includesInstanceProperty(finalSelectedValues).call(finalSelectedValues, o.value));
    }
    return _filterInstanceProperty(value).call(value, o => !_includesInstanceProperty(filteredValues).call(filteredValues, o.value));
  };
  const selectAllChanged = checked => {
    const newOptions = selectAllValues(checked);
    onChange(newOptions);
  };
  const handleSearchChange = e => {
    debouncedSearch(e.target.value);
    setSearchText(e.target.value);
    setFocusIndex(FocusType.SEARCH);
  };
  const handleClear = () => {
    var _a;
    setSearchTextForFilter('');
    setSearchText('');
    (_a = searchInputRef === null || searchInputRef === void 0 ? void 0 : searchInputRef.current) === null || _a === void 0 ? void 0 : _a.focus();
  };
  const handleItemClicked = index => {
    setFocusIndex(index);
  };
  // Arrow Key Navigation
  const handleKeyDown = e => {
    switch (e.code) {
      case KEY.ARROW_UP:
        updateFocus(-1);
        break;
      case KEY.ARROW_DOWN:
        updateFocus(1);
        break;
      default:
        return;
    }
    e.stopPropagation();
    e.preventDefault();
  };
  useKey([KEY.ARROW_DOWN, KEY.ARROW_UP], handleKeyDown, {
    target: listRef
  });
  const handleSearchFocus = () => {
    setFocusIndex(FocusType.SEARCH);
  };
  const handleOnCreateOption = () => __awaiter(void 0, void 0, void 0, function* () {
    let newOption = {
      label: searchText,
      value: searchText,
      __isNew__: true
    };
    // if custom `onCreateOption` is given then this will call this
    if (onCreateOption) {
      newOption = yield onCreateOption(searchText);
    }
    // adds created value to existing options
    setOptions([newOption, ...options]);
    handleClear();
    onChange([...value, newOption]);
  });
  const getFilteredOptions = () => __awaiter(void 0, void 0, void 0, function* () {
    return customFilterOptions ? yield customFilterOptions(options, searchTextForFilter) : simpleFilterOptions(options, searchTextForFilter);
  });
  const updateFocus = offset => {
    let newFocus = focusIndex + offset;
    newFocus = Math.max(0, newFocus);
    newFocus = Math.min(newFocus, options.length + Math.max(skipIndex - 1, 0));
    setFocusIndex(newFocus);
  };
  useEffect(() => {
    var _a, _b;
    (_b = (_a = listRef === null || listRef === void 0 ? void 0 : listRef.current) === null || _a === void 0 ? void 0 : _a.querySelector(`[tabIndex='${focusIndex}']`)) === null || _b === void 0 ? void 0 : _b.focus();
  }, [focusIndex]);
  const [isAllOptionSelected, hasSelectableOptions] = useMemo(() => {
    const filteredOptionsList = _filterInstanceProperty(filteredOptions).call(filteredOptions, o => !o.disabled);
    return [_everyInstanceProperty(filteredOptionsList).call(filteredOptionsList, o => _findIndexInstanceProperty(value).call(value, v => v.value === o.value) !== -1), filteredOptionsList.length !== 0];
    // eslint-disable-next-line
  }, [filteredOptions, value]);
  useEffect(() => {
    getFilteredOptions().then(setFilteredOptions);
  }, [searchTextForFilter, options]);
  const creationRef = useRef();
  useKey([KEY.ENTER], handleOnCreateOption, {
    target: creationRef
  });
  const showCreatable = isCreatable && searchText && !_someInstanceProperty(filteredOptions).call(filteredOptions, e => (e === null || e === void 0 ? void 0 : e.value) === searchText);
  return _jsxs(Styled.SelectPanelWrapper, _Object$assign({
    role: "listbox",
    ref: listRef
  }, {
    children: [!disableSearch && _jsxs(Styled.SearchArea, {
      children: [_jsx("input", {
        placeholder: t('search'),
        type: "text",
        "aria-describedby": t('search'),
        onChange: handleSearchChange,
        onFocus: handleSearchFocus,
        value: searchText,
        ref: searchInputRef,
        tabIndex: 0
      }), _jsx("button", _Object$assign({
        type: "button",
        hidden: !searchText,
        onClick: handleClear,
        "aria-label": t('clearSearch')
      }, {
        children: ClearIcon || _jsx(CrossIcon, {})
      }))]
    }), _jsxs(Styled.OptionsArea, {
      children: [hasSelectAll && hasSelectableOptions && _jsx(SelectItem, {
        tabIndex: skipIndex === 1 ? 0 : 1,
        checked: isAllOptionSelected,
        option: selectAllOption,
        onSelectionChanged: selectAllChanged,
        onClick: () => handleItemClicked(1),
        itemRenderer: ItemRenderer,
        disabled: disabled,
        isCheckboxShowing: isCheckboxShowing
      }), filteredOptions.length ? _jsx(SelectList, {
        isCheckboxShowing: isCheckboxShowing,
        skipIndex: skipIndex,
        options: filteredOptions,
        onClick: (_e, index) => handleItemClicked(index),
        closeOnChangedValue: closeOnChangedValue
      }) : showCreatable ? _jsx(Styled.SelectItemWrapper, _Object$assign({
        onClick: handleOnCreateOption,
        tabIndex: 1,
        ref: creationRef
      }, {
        children: `${t('create')} "${searchText}"`
      })) : _jsx(Styled.NoOptionsItem, {
        children: t('noOptions')
      })]
    })]
  }));
};
export default SelectPanel;